@@ -5,7 +5,7 @@
 %%% Created : 31 Oct 2005 by Magnus Henoch <henoch@dtek.chalmers.se>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -44,10 +44,11 @@ parse_request(#iq{type = set, lang = Lang, sub_el = SubEl, xmlns = ?NS_COMMANDS}
     Action = xml:get_tag_attr_s("action", SubEl),
     XData = find_xdata_el(SubEl),
     {xmlelement, _, _, AllEls} = SubEl,
-    if XData ->
-	    Others = lists:delete(XData, AllEls);
-       true ->
-	    Others = AllEls
+    Others = case XData of
+	false ->
+	    AllEls;
+	_ ->
+	    lists:delete(XData, AllEls)
     end,
 
     #adhoc_request{lang = Lang,
