@@ -5,7 +5,7 @@
 %%% Created : 16 Nov 2002 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -32,6 +32,7 @@
 
 %% External exports
 -export([start/2,
+	 stop/1,
 	 start_link/2,
 	 send_text/2,
 	 send_element/2,
@@ -66,6 +67,7 @@
 -record(state, {socket,
 		sockmod,
 		socket_monitor,
+		xml_socket,
 		streamid,
 		sasl_state,
 		access,
@@ -123,16 +125,12 @@
 
 -define(STREAM_TRAILER, "</stream:stream>").
 
--define(INVALID_NS_ERR,
-	xml:element_to_string(?SERR_INVALID_NAMESPACE)).
--define(INVALID_XML_ERR,
-	xml:element_to_string(?SERR_XML_NOT_WELL_FORMED)).
--define(HOST_UNKNOWN_ERR,
-	xml:element_to_string(?SERR_HOST_UNKNOWN)).
+-define(INVALID_NS_ERR, ?SERR_INVALID_NAMESPACE).
+-define(INVALID_XML_ERR, ?SERR_XML_NOT_WELL_FORMED).
+-define(HOST_UNKNOWN_ERR, ?SERR_HOST_UNKNOWN).
 -define(POLICY_VIOLATION_ERR(Lang, Text),
-	xml:element_to_string(?SERRT_POLICY_VIOLATION(Lang, Text))).
--define(INVALID_FROM,
-	xml:element_to_string(?SERR_INVALID_FROM)).
+	?SERRT_POLICY_VIOLATION(Lang, Text)).
+-define(INVALID_FROM, ?SERR_INVALID_FROM).
 
 
 %%%----------------------------------------------------------------------
@@ -151,6 +149,9 @@ socket_type() ->
 get_presence(FsmRef) ->
     gen_fsm:sync_send_all_state_event(FsmRef, {get_presence}, 1000).
 
+stop(FsmRef) ->
+    gen_fsm:send_event(FsmRef, closed).
+
 %%%----------------------------------------------------------------------
 %%% Callback functions from gen_fsm
 %%%----------------------------------------------------------------------
@@ -171,6 +172,11 @@ init([{SockMod, Socket}, Opts]) ->
 		 {value, {_, S}} -> S;
 		 _ -> none
 	     end,
+    XMLSocket =
+	case lists:keysearch(xml_socket, 1, Opts) of
+	    {value, {_, XS}} -> XS;
+	    _ -> false
+	end,
     Zlib = lists:member(zlib, Opts),
     StartTLS = lists:member(starttls, Opts),
     StartTLSRequired = lists:member(starttls_required, Opts),
@@ -186,7 +192,7 @@ init([{SockMod, Socket}, Opts]) ->
     case is_ip_blacklisted(IP) of
 	true ->
 	    ?INFO_MSG("Connection attempt from blacklisted IP: ~s",
-	              [jlib:ip_to_list(IP)]),
+		      [jlib:ip_to_list(IP)]),
 	    {stop, normal};
 	false ->
 	    Socket1 =
@@ -198,17 +204,19 @@ init([{SockMod, Socket}, Opts]) ->
 		end,
 	    SocketMonitor = SockMod:monitor(Socket1),
 	    {ok, wait_for_stream, #state{socket         = Socket1,
-					sockmod        = SockMod,
-					socket_monitor = SocketMonitor,
-					zlib           = Zlib,
-					tls            = TLS,
-					tls_required   = StartTLSRequired,
-					tls_enabled    = TLSEnabled,
-					tls_options    = TLSOpts,
-					streamid       = new_id(),
-					access         = Access,
-					shaper         = Shaper,
-					ip             = IP}, ?C2S_OPEN_TIMEOUT}
+					 sockmod        = SockMod,
+					 socket_monitor = SocketMonitor,
+					 xml_socket     = XMLSocket,
+					 zlib           = Zlib,
+					 tls            = TLS,
+					 tls_required   = StartTLSRequired,
+					 tls_enabled    = TLSEnabled,
+					 tls_options    = TLSOpts,
+					 streamid       = new_id(),
+					 access         = Access,
+					 shaper         = Shaper,
+					 ip             = IP},
+	     ?C2S_OPEN_TIMEOUT}
     end.
 
 %% Return list of all available resources of contacts,
@@ -225,9 +233,9 @@ get_subscribed(FsmRef) ->
 wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
     DefaultLang = case ?MYLANG of
 		      undefined ->
-			  " xml:lang='en'";
+			  "en";
 		      DL ->
-			  " xml:lang='" ++ DL ++ "'"
+			  DL
 		  end,
     case xml:get_attr_s("xmlns:stream", Attrs) of
 	?NS_STREAM ->
@@ -238,12 +246,7 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 		    change_shaper(StateData, jlib:make_jid("", Server, "")),
 		    case xml:get_attr_s("version", Attrs) of
 			"1.0" ->
-			    Header = io_lib:format(?STREAM_HEADER,
-						   [StateData#state.streamid,
-						    Server,
-						    " version='1.0'",
-						    DefaultLang]),
-			    send_text(StateData, Header),
+			    send_header(StateData, Server, "1.0", DefaultLang),
 			    case StateData#state.authenticated of
 				false ->
 				    SASLState =
@@ -256,6 +259,10 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 					  fun(U, P) ->
 						  ejabberd_auth:check_password_with_authmodule(
 						    U, Server, P)
+					  end,
+					  fun(U, P, D, DG) ->
+						  ejabberd_auth:check_password_with_authmodule(
+						    U, Server, P, D, DG)
 					  end),
 				    Mechs = lists:map(
 					      fun(S) ->
@@ -268,7 +275,8 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 				    Zlib = StateData#state.zlib,
 				    CompressFeature =
 					case Zlib andalso
-					    (SockMod == gen_tcp) of
+					    ((SockMod == gen_tcp) orelse
+					     (SockMod == tls)) of
 					    true ->
 						[{xmlelement, "compression",
 						  [{"xmlns", ?NS_FEATURE_COMPRESS}],
@@ -316,13 +324,15 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 				_ ->
 				    case StateData#state.resource of
 					"" ->
+					    RosterVersioningFeature = ejabberd_hooks:run_fold(roster_get_versioning_feature, Server, [], [Server]),
+				            StreamFeatures = [{xmlelement, "bind",
+						 [{"xmlns", ?NS_BIND}], []},
+						{xmlelement, "session",
+						 [{"xmlns", ?NS_SESSION}], []} | RosterVersioningFeature],
 					    send_element(
 					      StateData,
 					      {xmlelement, "stream:features", [],
-					       [{xmlelement, "bind",
-						 [{"xmlns", ?NS_BIND}], []},
-						{xmlelement, "session",
-						 [{"xmlns", ?NS_SESSION}], []}]}),
+					       StreamFeatures}),
 					    fsm_next_state(wait_for_bind,
 						       StateData#state{
 							 server = Server,
@@ -338,22 +348,18 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 				    end
 			    end;
 			_ ->
-			    Header = io_lib:format(
-				       ?STREAM_HEADER,
-				       [StateData#state.streamid, Server, "",
-					DefaultLang]),
+			    send_header(StateData, Server, "", DefaultLang),
 			    if
 				(not StateData#state.tls_enabled) and
 				StateData#state.tls_required ->
-				    send_text(StateData,
-					      Header ++
-					      ?POLICY_VIOLATION_ERR(
-						 Lang,
-						 "Use of STARTTLS required") ++
-					      ?STREAM_TRAILER),
+				    send_element(
+				      StateData,
+				      ?POLICY_VIOLATION_ERR(
+					 Lang,
+					 "Use of STARTTLS required")),
+				    send_trailer(StateData),
 				    {stop, normal, StateData};
 				true ->
-				    send_text(StateData, Header),
 				    fsm_next_state(wait_for_auth,
 						   StateData#state{
 						     server = Server,
@@ -361,20 +367,15 @@ wait_for_stream({xmlstreamstart, _Name, Attrs}, StateData) ->
 			    end
 		    end;
 		_ ->
-		    Header = io_lib:format(
-			       ?STREAM_HEADER,
-			       [StateData#state.streamid, ?MYNAME, "",
-				DefaultLang]),
-		    send_text(StateData,
-			      Header ++ ?HOST_UNKNOWN_ERR ++ ?STREAM_TRAILER),
+		    send_header(StateData, ?MYNAME, "", DefaultLang),
+		    send_element(StateData, ?HOST_UNKNOWN_ERR),
+		    send_trailer(StateData),
 		    {stop, normal, StateData}
 	    end;
 	_ ->
-	    Header = io_lib:format(
-		       ?STREAM_HEADER,
-		       [StateData#state.streamid, ?MYNAME, "", DefaultLang]),
-	    send_text(StateData,
-		      Header ++ ?INVALID_NS_ERR ++ ?STREAM_TRAILER),
+	    send_header(StateData, ?MYNAME, "", DefaultLang),
+	    send_element(StateData, ?INVALID_NS_ERR),
+	    send_trailer(StateData),
 	    {stop, normal, StateData}
     end;
 
@@ -382,18 +383,19 @@ wait_for_stream(timeout, StateData) ->
     {stop, normal, StateData};
 
 wait_for_stream({xmlstreamelement, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_stream({xmlstreamend, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_stream({xmlstreamerror, _}, StateData) ->
-    Header = io_lib:format(?STREAM_HEADER,
-			   ["none", ?MYNAME, " version='1.0'", ""]),
-    send_text(StateData,
-	      Header ++ ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_header(StateData, ?MYNAME, "1.0", ""),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_stream(closed, StateData) ->
@@ -442,14 +444,15 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 		(acl:match_rule(StateData#state.server,
 				StateData#state.access, JID) == allow) of
 		true ->
+                    DGen = fun(PW) ->
+                             sha:sha(StateData#state.streamid ++ PW) end,
 		    case ejabberd_auth:check_password_with_authmodule(
-			   U, StateData#state.server, P,
-			   StateData#state.streamid, D) of
+			   U, StateData#state.server, P, D, DGen) of
 			{true, AuthModule} ->
 			    ?INFO_MSG(
-			       "(~w) Accepted legacy authentication for ~s",
+			       "(~w) Accepted legacy authentication for ~s by ~p",
 			       [StateData#state.socket,
-				jlib:jid_to_string(JID)]),
+				jlib:jid_to_string(JID), AuthModule]),
 			    SID = {now(), self()},
 			    Conn = get_conn_type(StateData),
 			    Info = [{ip, StateData#state.ip}, {conn, Conn},
@@ -474,8 +477,8 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 				  privacy_get_user_list, StateData#state.server,
 				  #userlist{},
 				  [U, StateData#state.server]),
-			    fsm_next_state(session_established,
-					   StateData#state{
+                            NewStateData =
+                                StateData#state{
 					     user = U,
 					     resource = R,
 					     jid = JID,
@@ -484,7 +487,9 @@ wait_for_auth({xmlstreamelement, El}, StateData) ->
 					     auth_module = AuthModule,
 					     pres_f = ?SETS:from_list(Fs1),
 					     pres_t = ?SETS:from_list(Ts1),
-					     privacy_list = PrivList});
+					     privacy_list = PrivList},
+			    fsm_next_state_pack(session_established,
+                                                NewStateData);
 			_ ->
 			    ?INFO_MSG(
 			       "(~w) Failed legacy authentication for ~s",
@@ -524,11 +529,12 @@ wait_for_auth(timeout, StateData) ->
     {stop, normal, StateData};
 
 wait_for_auth({xmlstreamend, _Name}, StateData) ->
-    send_text(StateData, ?STREAM_TRAILER),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_auth({xmlstreamerror, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_auth(closed, StateData) ->
@@ -556,12 +562,14 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 				 {xmlelement, "success",
 				  [{"xmlns", ?NS_SASL}], []}),
 		    U = xml:get_attr_s(username, Props),
-		    ?INFO_MSG("(~w) Accepted authentication for ~s",
-			      [StateData#state.socket, U]),
+		    AuthModule = xml:get_attr_s(auth_module, Props),
+		    ?INFO_MSG("(~w) Accepted authentication for ~s by ~p",
+			      [StateData#state.socket, U, AuthModule]),
 		    fsm_next_state(wait_for_stream,
 				   StateData#state{
 				     streamid = new_id(),
 				     authenticated = true,
+				     auth_module = AuthModule,
 				     user = U });
 		{continue, ServerOut, NewSASLState} ->
 		    send_element(StateData,
@@ -613,7 +621,8 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 					   tls_enabled = true
 					  });
 	{?NS_COMPRESS, "compress"} when Zlib == true,
-					SockMod == gen_tcp ->
+					((SockMod == gen_tcp) or
+					 (SockMod == tls)) ->
 	    case xml:get_subtag(El, "method") of
 		false ->
 		    send_element(StateData,
@@ -648,10 +657,10 @@ wait_for_feature_request({xmlstreamelement, El}, StateData) ->
 	    if
 		(SockMod == gen_tcp) and TLSRequired ->
 		    Lang = StateData#state.lang,
-		    send_text(StateData, ?POLICY_VIOLATION_ERR(
-					    Lang,
-					    "Use of STARTTLS required") ++
-					 ?STREAM_TRAILER),
+		    send_element(StateData, ?POLICY_VIOLATION_ERR(
+					       Lang,
+					       "Use of STARTTLS required")),
+		    send_trailer(StateData),
 		    {stop, normal, StateData};
 		true ->
 		    process_unauthenticated_stanza(StateData, El),
@@ -663,11 +672,12 @@ wait_for_feature_request(timeout, StateData) ->
     {stop, normal, StateData};
 
 wait_for_feature_request({xmlstreamend, _Name}, StateData) ->
-    send_text(StateData, ?STREAM_TRAILER),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_feature_request({xmlstreamerror, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_feature_request(closed, StateData) ->
@@ -689,8 +699,8 @@ wait_for_sasl_response({xmlstreamelement, El}, StateData) ->
 				  [{"xmlns", ?NS_SASL}], []}),
 		    U = xml:get_attr_s(username, Props),
 		    AuthModule = xml:get_attr_s(auth_module, Props),
-		    ?INFO_MSG("(~w) Accepted authentication for ~s",
-			      [StateData#state.socket, U]),
+		    ?INFO_MSG("(~w) Accepted authentication for ~s by ~p",
+			      [StateData#state.socket, U, AuthModule]),
 		    fsm_next_state(wait_for_stream,
 				   StateData#state{
 				     streamid = new_id(),
@@ -731,11 +741,12 @@ wait_for_sasl_response(timeout, StateData) ->
     {stop, normal, StateData};
 
 wait_for_sasl_response({xmlstreamend, _Name}, StateData) ->
-    send_text(StateData, ?STREAM_TRAILER),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_sasl_response({xmlstreamerror, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_sasl_response(closed, StateData) ->
@@ -762,6 +773,15 @@ wait_for_bind({xmlstreamelement, El}, StateData) ->
 		    fsm_next_state(wait_for_bind, StateData);
 		_ ->
 		    JID = jlib:make_jid(U, StateData#state.server, R),
+		    %%Server = StateData#state.server,
+		    %%RosterVersioningFeature =
+		    %%	ejabberd_hooks:run_fold(
+		    %%  roster_get_versioning_feature, Server, [], [Server]),
+	            %%StreamFeatures = [{xmlelement, "session",
+		    %%		       [{"xmlns", ?NS_SESSION}], []} |
+		    %%		      RosterVersioningFeature],
+		    %%send_element(StateData, {xmlelement, "stream:features",
+		    %%			     [], StreamFeatures}),
 		    Res = IQ#iq{type = result,
 				sub_el = [{xmlelement, "bind",
 					   [{"xmlns", ?NS_BIND}],
@@ -780,11 +800,12 @@ wait_for_bind(timeout, StateData) ->
     {stop, normal, StateData};
 
 wait_for_bind({xmlstreamend, _Name}, StateData) ->
-    send_text(StateData, ?STREAM_TRAILER),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_bind({xmlstreamerror, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_bind(closed, StateData) ->
@@ -804,12 +825,6 @@ wait_for_session({xmlstreamelement, El}, StateData) ->
 		    ?INFO_MSG("(~w) Opened session for ~s",
 			      [StateData#state.socket,
 			       jlib:jid_to_string(JID)]),
-		    SID = {now(), self()},
-		    Conn = get_conn_type(StateData),
-		    Info = [{ip, StateData#state.ip}, {conn, Conn},
-			    {auth_module, StateData#state.auth_module}],
-		    ejabberd_sm:open_session(
-		      SID, U, StateData#state.server, R, Info),
 		    Res = jlib:make_result_iq_reply(El),
 		    send_element(StateData, Res),
 		    change_shaper(StateData, JID),
@@ -826,14 +841,24 @@ wait_for_session({xmlstreamelement, El}, StateData) ->
 			  privacy_get_user_list, StateData#state.server,
 			  #userlist{},
 			  [U, StateData#state.server]),
-		    fsm_next_state(session_established,
-				   StateData#state{
+		    SID = {now(), self()},
+		    Conn = get_conn_type(StateData),
+		    Info = [{ip, StateData#state.ip}, {conn, Conn},
+			    {auth_module, StateData#state.auth_module}],
+		    ejabberd_sm:open_session(
+		      SID, U, StateData#state.server, R, Info),
+                    NewStateData =
+                        StateData#state{
 				     sid = SID,
 				     conn = Conn,
 				     pres_f = ?SETS:from_list(Fs1),
 				     pres_t = ?SETS:from_list(Ts1),
-				     privacy_list = PrivList});
+				     privacy_list = PrivList},
+		    fsm_next_state_pack(session_established,
+                                        NewStateData);
 		_ ->
+		    ejabberd_hooks:run(forbidden_session_hook,
+				       StateData#state.server, [JID]),
 		    ?INFO_MSG("(~w) Forbidden session for ~s",
 			      [StateData#state.socket,
 			       jlib:jid_to_string(JID)]),
@@ -849,11 +874,12 @@ wait_for_session(timeout, StateData) ->
     {stop, normal, StateData};
 
 wait_for_session({xmlstreamend, _Name}, StateData) ->
-    send_text(StateData, ?STREAM_TRAILER),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_session({xmlstreamerror, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 wait_for_session(closed, StateData) ->
@@ -865,7 +891,8 @@ session_established({xmlstreamelement, El}, StateData) ->
     % Check 'from' attribute in stanza RFC 3920 Section 9.1.2
     case check_from(El, FromJID) of
 	'invalid-from' ->
-	    send_text(StateData, ?INVALID_FROM ++ ?STREAM_TRAILER),
+	    send_element(StateData, ?INVALID_FROM),
+	    send_trailer(StateData),
 	    {stop, normal, StateData};
 	_NewEl ->
 	    session_established2(El, StateData)
@@ -881,16 +908,24 @@ session_established(timeout, StateData) ->
     fsm_next_state(session_established, StateData);
 
 session_established({xmlstreamend, _Name}, StateData) ->
-    send_text(StateData, ?STREAM_TRAILER),
+    send_trailer(StateData),
+    {stop, normal, StateData};
+
+session_established({xmlstreamerror, "XML stanza is too big" = E}, StateData) ->
+    send_element(StateData, ?POLICY_VIOLATION_ERR(StateData#state.lang, E)),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 session_established({xmlstreamerror, _}, StateData) ->
-    send_text(StateData, ?INVALID_XML_ERR ++ ?STREAM_TRAILER),
+    send_element(StateData, ?INVALID_XML_ERR),
+    send_trailer(StateData),
     {stop, normal, StateData};
 
 session_established(closed, StateData) ->
     {stop, normal, StateData}.
 
+%% Process packets sent by user (coming from user on c2s XMPP
+%% connection)
 session_established2(El, StateData) ->
     {xmlelement, Name, Attrs, _Els} = El,
     User = StateData#state.user,
@@ -933,6 +968,10 @@ session_established2(El, StateData) ->
 				       Server,
 				       NewEl,
 				       [User, Server]),
+			ejabberd_hooks:run(
+			  user_send_packet,
+			  Server,
+			  [FromJID, ToJID, PresenceEl]),
 			case ToJID of
 			    #jid{user = User,
 				 server = Server,
@@ -963,7 +1002,8 @@ session_established2(El, StateData) ->
 			ejabberd_hooks:run(user_send_packet,
 					   Server,
 					   [FromJID, ToJID, NewEl]),
-			ejabberd_router:route(FromJID, ToJID, NewEl),
+			check_privacy_route(FromJID, StateData, FromJID,
+					    ToJID, NewEl),
 			StateData;
 		    _ ->
 			StateData
@@ -1039,11 +1079,11 @@ handle_info({send_text, Text}, StateName, StateData) ->
     fsm_next_state(StateName, StateData);
 handle_info(replaced, _StateName, StateData) ->
     Lang = StateData#state.lang,
-    send_text(StateData,
-	      xml:element_to_string(
-		?SERRT_CONFLICT(Lang, "Replaced by new connection"))
-	      ++ ?STREAM_TRAILER),
+    send_element(StateData,
+		 ?SERRT_CONFLICT(Lang, "Replaced by new connection")),
+    send_trailer(StateData),
     {stop, normal, StateData#state{authenticated = replaced}};
+%% Process Packets that are to be send to the user
 handle_info({route, From, To, Packet}, StateName, StateData) ->
     {xmlelement, Name, Attrs, Els} = Packet,
     {Pass, NewAttrs, NewState} =
@@ -1114,16 +1154,6 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 			    allow ->
 				LFrom = jlib:jid_tolower(From),
 				LBFrom = jlib:jid_remove_resource(LFrom),
-				%% Note contact availability
-				case xml:get_attr_s("type", Attrs) of
-				    "unavailable" -> 
-					%mod_caps:clear_caps(From);
-					% caps clear disabled cause it breaks things
-					ok;
-				    _ -> 
-					Caps = mod_caps:read_caps(Els),
-					mod_caps:note_caps(StateData#state.server, From, Caps)
-				end,
 				case ?SETS:is_element(
 					LFrom, StateData#state.pres_a) orelse
 				    ?SETS:is_element(
@@ -1198,7 +1228,7 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 	    "iq" ->
 		IQ = jlib:iq_query_info(Packet),
 		case IQ of
-		    #iq{xmlns = ?NS_VCARD} ->
+		    #iq{xmlns = ?NS_VCARD} when (To#jid.luser == "") or (To#jid.lresource == "") ->
 			Host = StateData#state.server,
 			case ets:lookup(sm_iqtable, {?NS_VCARD, Host}) of
 			    [{_, Module, Function, Opts}] ->
@@ -1249,15 +1279,17 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 	end,
     if
 	Pass == exit ->
-	    catch send_text(StateData, ?STREAM_TRAILER),
+	    %% When Pass==exit, NewState contains a string instead of a #state{}
+	    Lang = StateData#state.lang,
+	    send_element(StateData, ?SERRT_CONFLICT(Lang, NewState)),
+	    send_trailer(StateData),
 	    {stop, normal, StateData};
 	Pass ->
 	    Attrs2 = jlib:replace_from_to_attrs(jlib:jid_to_string(From),
 						jlib:jid_to_string(To),
 						NewAttrs),
 	    FixedPacket = {xmlelement, Name, Attrs2, Els},
-	    Text = xml:element_to_string(FixedPacket),
-	    send_text(StateData, Text),
+	    send_element(StateData, FixedPacket),
 	    ejabberd_hooks:run(user_receive_packet,
 			       StateData#state.server,
 			       [StateData#state.jid, From, To, FixedPacket]),
@@ -1270,6 +1302,19 @@ handle_info({route, From, To, Packet}, StateName, StateData) ->
 handle_info({'DOWN', Monitor, _Type, _Object, _Info}, _StateName, StateData)
   when Monitor == StateData#state.socket_monitor ->
     {stop, normal, StateData};
+handle_info(system_shutdown, StateName, StateData) ->
+    case StateName of
+       wait_for_stream ->
+           send_header(StateData, ?MYNAME, "1.0", "en"),
+           send_element(StateData, ?SERR_SYSTEM_SHUTDOWN),
+           send_trailer(StateData),
+           ok;
+       _ ->
+           send_element(StateData, ?SERR_SYSTEM_SHUTDOWN),
+           send_trailer(StateData),
+           ok
+    end,
+    {stop, normal, StateData};
 handle_info(Info, StateName, StateData) ->
     ?ERROR_MSG("Unexpected info: ~p", [Info]),
     fsm_next_state(StateName, StateData).
@@ -1352,9 +1397,60 @@ send_text(StateData, Text) ->
     ?DEBUG("Send XML on stream = ~p", [lists:flatten(Text)]),
     (StateData#state.sockmod):send(StateData#state.socket, Text).
 
+send_element(StateData, El) when StateData#state.xml_socket ->
+    (StateData#state.sockmod):send_xml(StateData#state.socket,
+				       {xmlstreamelement, El});
 send_element(StateData, El) ->
     send_text(StateData, xml:element_to_string(El)).
 
+send_header(StateData, Server, Version, Lang)
+  when StateData#state.xml_socket ->
+    VersionAttr =
+	case Version of
+	    "" -> [];
+	    _ -> [{"version", Version}]
+	end,
+    LangAttr =
+	case Lang of
+	    "" -> [];
+	    _ -> [{"xml:lang", Lang}]
+	end,
+    Header =
+	{xmlstreamstart,
+	 "stream:stream",
+	 VersionAttr ++
+	 LangAttr ++
+	 [{"xmlns", "jabber:client"},
+	  {"xmlns:stream", "http://etherx.jabber.org/streams"},
+	  {"id", StateData#state.streamid},
+	  {"from", Server}]},
+    (StateData#state.sockmod):send_xml(
+      StateData#state.socket, Header);
+send_header(StateData, Server, Version, Lang) ->
+    VersionStr =
+	case Version of
+	    "" -> "";
+	    _ -> [" version='", Version, "'"]
+	end,
+    LangStr =
+	case Lang of
+	    "" -> "";
+	    _ -> [" xml:lang='", Lang, "'"]
+	end,
+    Header = io_lib:format(?STREAM_HEADER,
+			   [StateData#state.streamid,
+			    Server,
+			    VersionStr,
+			    LangStr]),
+    send_text(StateData, Header).
+
+send_trailer(StateData) when StateData#state.xml_socket ->
+    (StateData#state.sockmod):send_xml(
+      StateData#state.socket,
+      {xmlstreamend, "stream:stream"});
+send_trailer(StateData) ->
+    send_text(StateData, ?STREAM_TRAILER).
+
 
 new_id() ->
     randoms:get_string().
@@ -1390,11 +1486,18 @@ get_auth_tags([_ | L], U, P, D, R) ->
 get_auth_tags([], U, P, D, R) ->
     {U, P, D, R}.
 
+%% Copied from ejabberd_socket.erl
+-record(socket_state, {sockmod, socket, receiver}).
+
 get_conn_type(StateData) ->
     case (StateData#state.sockmod):get_sockmod(StateData#state.socket) of
     gen_tcp -> c2s;
     tls -> c2s_tls;
-    ejabberd_zlib -> c2s_compressed;
+    ejabberd_zlib ->
+	case ejabberd_zlib:get_sockmod((StateData#state.socket)#socket_state.socket) of
+	    gen_tcp -> c2s_compressed;
+	    tls -> c2s_compressed_tls
+	end;
     ejabberd_http_poll -> http_poll;
     ejabberd_http_bind -> http_bind;
     _ -> unknown
@@ -1422,7 +1525,13 @@ process_presence_probe(From, To, StateData) ->
 		andalso ?SETS:is_element(LFrom, StateData#state.pres_a),
 	    if
 		Cond1 ->
-		    Packet = StateData#state.pres_last,
+		    Timestamp = StateData#state.pres_timestamp,
+		    Packet = xml:append_subtags(
+			       StateData#state.pres_last,
+			       %% To is the one sending the presence (the target of the probe)
+			       [jlib:timestamp_to_xml(Timestamp, utc, To, ""),
+				%% TODO: Delete the next line once XEP-0091 is Obsolete
+				jlib:timestamp_to_xml(Timestamp)]),
 		    case ejabberd_hooks:run_fold(
 			   privacy_check_packet, StateData#state.server,
 			   allow,
@@ -1454,6 +1563,7 @@ process_presence_probe(From, To, StateData) ->
 	    end
     end.
 
+%% User updates his presence (non-directed presence packet)
 presence_update(From, Packet, StateData) ->
     {xmlelement, _Name, Attrs, _Els} = Packet,
     case xml:get_attr_s("type", Attrs) of
@@ -1475,10 +1585,13 @@ presence_update(From, Packet, StateData) ->
 	    presence_broadcast(StateData, From, StateData#state.pres_a, Packet),
 	    presence_broadcast(StateData, From, StateData#state.pres_i, Packet),
 	    StateData#state{pres_last = undefined,
+			    pres_timestamp = undefined,
 			    pres_a = ?SETS:new(),
 			    pres_i = ?SETS:new(),
 			    pres_invis = false};
 	"invisible" ->
+	    NewPriority = get_priority_from_presence(Packet),
+	    update_priority(NewPriority, Packet, StateData),
 	    NewState =
 		if
 		    not StateData#state.pres_invis ->
@@ -1489,6 +1602,7 @@ presence_update(From, Packet, StateData) ->
 					   StateData#state.pres_i,
 					   Packet),
 			S1 = StateData#state{pres_last = undefined,
+					     pres_timestamp = undefined,
 					     pres_a = ?SETS:new(),
 					     pres_i = ?SETS:new(),
 					     pres_invis = true},
@@ -1517,6 +1631,7 @@ presence_update(From, Packet, StateData) ->
 				  get_priority_from_presence(OldPresence)
 			  end,
 	    NewPriority = get_priority_from_presence(Packet),
+	    Timestamp = calendar:now_to_universal_time(now()),
 	    update_priority(NewPriority, Packet, StateData),
 	    FromUnavail = (StateData#state.pres_last == undefined) or
 		StateData#state.pres_invis,
@@ -1535,7 +1650,8 @@ presence_update(From, Packet, StateData) ->
 			end,
 			presence_broadcast_first(
 			  From, StateData#state{pres_last = Packet,
-						pres_invis = false
+						pres_invis = false,
+						pres_timestamp = Timestamp
 					       }, Packet);
 		    true ->
 			presence_broadcast_to_trusted(StateData,
@@ -1549,12 +1665,14 @@ presence_update(From, Packet, StateData) ->
 				ok
 			end,
 			StateData#state{pres_last = Packet,
-					pres_invis = false
+					pres_invis = false,
+					pres_timestamp = Timestamp
 				       }
 		end,
 	    NewState
     end.
 
+%% User sends a directed presence packet
 presence_track(From, To, Packet, StateData) ->
     {xmlelement, _Name, Attrs, _Els} = Packet,
     LTo = jlib:jid_tolower(To),
@@ -1562,13 +1680,13 @@ presence_track(From, To, Packet, StateData) ->
     Server = StateData#state.server,
     case xml:get_attr_s("type", Attrs) of
 	"unavailable" ->
-	    ejabberd_router:route(From, To, Packet),
+	    check_privacy_route(From, StateData, From, To, Packet),
 	    I = remove_element(LTo, StateData#state.pres_i),
 	    A = remove_element(LTo, StateData#state.pres_a),
 	    StateData#state{pres_i = I,
 			    pres_a = A};
 	"invisible" ->
-	    ejabberd_router:route(From, To, Packet),
+	    check_privacy_route(From, StateData, From, To, Packet),
 	    I = ?SETS:add_element(LTo, StateData#state.pres_i),
 	    A = remove_element(LTo, StateData#state.pres_a),
 	    StateData#state{pres_i = I,
@@ -1577,52 +1695,59 @@ presence_track(From, To, Packet, StateData) ->
 	    ejabberd_hooks:run(roster_out_subscription,
 			       Server,
 			       [User, Server, To, subscribe]),
-	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
+	    check_privacy_route(From, StateData, jlib:jid_remove_resource(From),
+				To, Packet),
 	    StateData;
 	"subscribed" ->
 	    ejabberd_hooks:run(roster_out_subscription,
 			       Server,
 			       [User, Server, To, subscribed]),
-	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
+	    check_privacy_route(From, StateData, jlib:jid_remove_resource(From),
+				To, Packet),
 	    StateData;
 	"unsubscribe" ->
 	    ejabberd_hooks:run(roster_out_subscription,
 			       Server,
 			       [User, Server, To, unsubscribe]),
-	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
+	    check_privacy_route(From, StateData, jlib:jid_remove_resource(From),
+				To, Packet),
 	    StateData;
 	"unsubscribed" ->
 	    ejabberd_hooks:run(roster_out_subscription,
 			       Server,
 			       [User, Server, To, unsubscribed]),
-	    ejabberd_router:route(jlib:jid_remove_resource(From), To, Packet),
+	    check_privacy_route(From, StateData, jlib:jid_remove_resource(From),
+				To, Packet),
 	    StateData;
 	"error" ->
-	    ejabberd_router:route(From, To, Packet),
+	    check_privacy_route(From, StateData, From, To, Packet),
 	    StateData;
 	"probe" ->
-	    ejabberd_router:route(From, To, Packet),
+	    check_privacy_route(From, StateData, From, To, Packet),
 	    StateData;
 	_ ->
-	    case ejabberd_hooks:run_fold(
-		   privacy_check_packet, StateData#state.server,
-		   allow,
-		   [StateData#state.user,
-		    StateData#state.server,
-		    StateData#state.privacy_list,
-		    {From, To, Packet},
-		    out]) of
-		deny ->
-		    ok;
-		allow ->
-		    ejabberd_router:route(From, To, Packet)
-	    end,
+	    check_privacy_route(From, StateData, From, To, Packet),
 	    I = remove_element(LTo, StateData#state.pres_i),
 	    A = ?SETS:add_element(LTo, StateData#state.pres_a),
 	    StateData#state{pres_i = I,
 			    pres_a = A}
     end.
 
+check_privacy_route(From, StateData, FromRoute, To, Packet) ->
+    case ejabberd_hooks:run_fold(
+	   privacy_check_packet, StateData#state.server,
+	   allow,
+	   [StateData#state.user,
+	    StateData#state.server,
+	    StateData#state.privacy_list,
+	    {From, To, Packet},
+	    out]) of
+	deny ->
+	    ok;
+	allow ->
+	    ejabberd_router:route(FromRoute, To, Packet)
+    end.
+
 %% Check if privacy rules allow this delivery
 is_privacy_allow(From, To, Packet, PrivacyList) ->
     User = To#jid.user,
@@ -1874,7 +1999,7 @@ resend_offline_messages(#state{user = User,
 				 Server,
 				 [],
 				 [User, Server]) of
-	Rs when list(Rs) ->
+	Rs when is_list(Rs) ->
 	    lists:foreach(
 	      fun({route,
 		   From, To, {xmlelement, Name, Attrs, Els} = Packet}) ->
@@ -1970,6 +2095,17 @@ peerip(SockMod, Socket) ->
 	_ -> undefined
     end.
 
+%% fsm_next_state_pack: Pack the StateData structure to improve
+%% sharing.
+fsm_next_state_pack(StateName, StateData) ->
+    fsm_next_state_gc(StateName, pack(StateData)).
+
+%% fsm_next_state_gc: Garbage collect the process heap to make use of
+%% the newly packed StateData structure.
+fsm_next_state_gc(StateName, PackedStateData) ->
+    erlang:garbage_collect(),
+    fsm_next_state(StateName, PackedStateData).
+
 %% fsm_next_state: Generate the next_state FSM tuple with different
 %% timeout, depending on the future state
 fsm_next_state(session_established, StateData) ->
@@ -1985,6 +2121,8 @@ fsm_reply(Reply, StateName, StateData) ->
     {reply, Reply, StateName, StateData, ?C2S_OPEN_TIMEOUT}.
 
 %% Used by c2s blacklist plugins
+is_ip_blacklisted(undefined) ->
+    false;
 is_ip_blacklisted({IP,_Port}) ->
     ejabberd_hooks:run_fold(check_bl_c2s, false, [IP]).
 
@@ -2014,3 +2152,52 @@ check_from(El, FromJID) ->
 		    end
 	    end
     end.
+
+%%%----------------------------------------------------------------------
+%%% JID Set memory footprint reduction code
+%%%----------------------------------------------------------------------
+
+%% Try to reduce the heap footprint of the four presence sets
+%% by ensuring that we re-use strings and Jids wherever possible.
+pack(S = #state{pres_a=A,
+                pres_i=I,
+                pres_f=F,
+                pres_t=T}) ->
+    {NewA, Pack1} = pack_jid_set(A, gb_trees:empty()),
+    {NewI, Pack2} = pack_jid_set(I, Pack1),
+    {NewF, Pack3} = pack_jid_set(F, Pack2),
+    {NewT, _Pack4} = pack_jid_set(T, Pack3),
+    %% Throw away Pack4 so that if we delete references to
+    %% Strings or Jids in any of the sets there will be
+    %% no live references for the GC to find.
+    S#state{pres_a=NewA,
+            pres_i=NewI,
+            pres_f=NewF,
+            pres_t=NewT}.
+
+pack_jid_set(Set, Pack) ->
+    Jids = ?SETS:to_list(Set),
+    {PackedJids, NewPack} = pack_jids(Jids, Pack, []),
+    {?SETS:from_list(PackedJids), NewPack}.
+
+pack_jids([], Pack, Acc) -> {Acc, Pack};
+pack_jids([{U,S,R}=Jid | Jids], Pack, Acc) ->
+    case gb_trees:lookup(Jid, Pack) of
+        {value, PackedJid} ->
+            pack_jids(Jids, Pack, [PackedJid | Acc]);
+        none ->
+            {NewU, Pack1} = pack_string(U, Pack),
+            {NewS, Pack2} = pack_string(S, Pack1),
+            {NewR, Pack3} = pack_string(R, Pack2),
+            NewJid = {NewU, NewS, NewR},
+            NewPack = gb_trees:insert(NewJid, NewJid, Pack3),
+            pack_jids(Jids, NewPack, [NewJid | Acc])
+    end.
+
+pack_string(String, Pack) ->
+    case gb_trees:lookup(String, Pack) of
+        {value, PackedString} ->
+            {PackedString, Pack};
+        none ->
+            {String, gb_trees:insert(String, String, Pack)}
+    end.
