@@ -5,7 +5,7 @@
 %%% Created : 22 Aug 2005 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -131,7 +131,14 @@ export_offline(Server, Output) ->
 	      NewPacket = {xmlelement, Name, Attrs2,
 			   Els ++
 			   [jlib:timestamp_to_xml(
-			      calendar:now_to_universal_time(TimeStamp))]},
+			      calendar:now_to_universal_time(TimeStamp),
+			      utc,
+			      jlib:make_jid("", Server, ""),
+			      "Offline Storage"),
+			    %% TODO: Delete the next three lines once XEP-0091 is Obsolete
+			    jlib:timestamp_to_xml(
+			      calendar:now_to_universal_time(
+				TimeStamp))]},
 	      XML =
 		  ejabberd_odbc:escape(
 		    lists:flatten(
