@@ -5,7 +5,7 @@
 %%% Created : 19 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -544,7 +544,7 @@ get_local_items({global, _Host}, ["running nodes", ENode], Server, Lang) ->
      [?NODE("Database", "running nodes/" ++ ENode ++ "/DB"),
       ?NODE("Modules", "running nodes/" ++ ENode ++ "/modules"),
       ?NODE("Backup Management", "running nodes/" ++ ENode ++ "/backup"),
-      ?NODE("Import Users From jabberd 1.4 Spool Files",
+      ?NODE("Import Users From jabberd14 Spool Files",
 	    "running nodes/" ++ ENode ++ "/import"),
       ?NODE("Restart Service", "running nodes/" ++ ENode ++ "/restart"),
       ?NODE("Shut Down Service", "running nodes/" ++ ENode ++ "/shutdown")
@@ -994,7 +994,7 @@ get_form(_Host, ["running nodes", ENode, "import", "file"], Lang) ->
 	        {xmlelement, "instructions", [],
 	         [{xmlcdata,
 	           ?T(
-		      Lang, "Enter path to jabberd1.4 spool file")}]},
+		      Lang, "Enter path to jabberd14 spool file")}]},
 		?XFIELD("text-single", "Path to File", "path", "")
 	       ]}]};
 
@@ -1008,7 +1008,7 @@ get_form(_Host, ["running nodes", ENode, "import", "dir"], Lang) ->
 	        {xmlelement, "instructions", [],
 	         [{xmlcdata,
 	           ?T(
-		      Lang, "Enter path to jabberd1.4 spool dir")}]},
+		      Lang, "Enter path to jabberd14 spool dir")}]},
 		?XFIELD("text-single", "Path to Dir", "path", "")
 	       ]}]};
 
@@ -1433,7 +1433,7 @@ set_form(_From, _Host, ["running nodes", ENode, "backup", "textfile"], _Lang, XD
 		false ->
 		    {error, ?ERR_BAD_REQUEST};
 		{value, {_, [String]}} ->
-		    case rpc:call(Node, ejabberd_ctl, dump_to_textfile, [String]) of
+		    case rpc:call(Node, ejabberd_admin, dump_to_textfile, [String]) of
 			{badrpc, _Reason} ->
 			    {error, ?ERR_INTERNAL_SERVER_ERROR};
 			{error, _Reason} ->
