@@ -5,7 +5,7 @@
 %%% Created :  5 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -32,13 +32,16 @@
 -export([count_offline_messages/2]).
 
 -export([start/2,
-	 init/2,
+	 loop/2,
 	 stop/1,
 	 store_packet/3,
 	 pop_offline_messages/3,
+	 get_sm_features/5,
 	 remove_user/2,
+	 get_queue_length/2,
 	 webadmin_page/3,
-	 webadmin_user/4]).
+	 webadmin_user/4,
+	 webadmin_user_parse_query/5]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
@@ -50,6 +53,9 @@
 -define(PROCNAME, ejabberd_offline).
 -define(OFFLINE_TABLE_LOCK_THRESHOLD, 1000).
 
+%% default value for the maximum number of user messages
+-define(MAX_USER_MESSAGES, infinity).
+
 start(Host, Opts) ->
     ejabberd_hooks:add(offline_message_hook, Host,
 		       ?MODULE, store_packet, 50),
@@ -59,26 +65,27 @@ start(Host, Opts) ->
 		       ?MODULE, remove_user, 50),
     ejabberd_hooks:add(anonymous_purge_hook, Host,
 		       ?MODULE, remove_user, 50),
+    ejabberd_hooks:add(disco_sm_features, Host,
+		       ?MODULE, get_sm_features, 50),
+    ejabberd_hooks:add(disco_local_features, Host,
+		       ?MODULE, get_sm_features, 50),
     ejabberd_hooks:add(webadmin_page_host, Host,
 		       ?MODULE, webadmin_page, 50),
     ejabberd_hooks:add(webadmin_user, Host,
 		       ?MODULE, webadmin_user, 50),
-    MaxOfflineMsgs = gen_mod:get_opt(user_max_messages, Opts, infinity),
+    ejabberd_hooks:add(webadmin_user_parse_query, Host,
+                       ?MODULE, webadmin_user_parse_query, 50),
+    AccessMaxOfflineMsgs = gen_mod:get_opt(access_max_user_messages, Opts, max_user_offline_messages),
     register(gen_mod:get_module_proc(Host, ?PROCNAME),
-	     spawn(?MODULE, init, [Host, MaxOfflineMsgs])).
+	     spawn(?MODULE, loop, [Host, AccessMaxOfflineMsgs])).
 
-%% MaxOfflineMsgs is either infinity of integer > 0
-init(Host, infinity) ->
-    loop(Host, infinity);
-init(Host, MaxOfflineMsgs) 
-  when integer(MaxOfflineMsgs), MaxOfflineMsgs > 0 ->
-    loop(Host, MaxOfflineMsgs).
-
-loop(Host, MaxOfflineMsgs) ->
+loop(Host, AccessMaxOfflineMsgs) ->
     receive
 	#offline_msg{user = User} = Msg ->
 	    Msgs = receive_all(User, [Msg]),
 	    Len = length(Msgs),
+	    MaxOfflineMsgs = get_max_user_messages(AccessMaxOfflineMsgs,
+						   User, Host),
 
 	    %% Only count existing messages if needed:
 	    Count = if MaxOfflineMsgs =/= infinity ->
@@ -106,6 +113,13 @@ loop(Host, MaxOfflineMsgs) ->
 						Els ++
 						[jlib:timestamp_to_xml(
 						   calendar:now_to_universal_time(
+					     M#offline_msg.timestamp),
+					   utc,
+					   jlib:make_jid("", Host, ""),
+					   "Offline Storage"),
+					 %% TODO: Delete the next three lines once XEP-0091 is Obsolete
+					 jlib:timestamp_to_xml( 
+					   calendar:now_to_universal_time(
 						     M#offline_msg.timestamp))]},
 				      XML =
 					  ejabberd_odbc:escape(
@@ -122,9 +136,18 @@ loop(Host, MaxOfflineMsgs) ->
 			    ok
 		    end
 	    end,
-	    loop(Host, MaxOfflineMsgs);
+	    loop(Host, AccessMaxOfflineMsgs);
 	_ ->
-	    loop(Host, MaxOfflineMsgs)
+	    loop(Host, AccessMaxOfflineMsgs)
+    end.
+
+%% Function copied from ejabberd_sm.erl:
+get_max_user_messages(AccessRule, LUser, Host) ->
+    case acl:match_rule(
+	   Host, AccessRule, jlib:make_jid(LUser, Host, "")) of
+	Max when is_integer(Max) -> Max;
+	infinity -> infinity;
+	_ -> ?MAX_USER_MESSAGES
     end.
 
 receive_all(Username, Msgs) ->
@@ -145,20 +168,39 @@ stop(Host) ->
 			  ?MODULE, remove_user, 50),
     ejabberd_hooks:delete(anonymous_purge_hook, Host,
 			  ?MODULE, remove_user, 50),
+    ejabberd_hooks:delete(disco_sm_features, Host, ?MODULE, get_sm_features, 50),
+    ejabberd_hooks:delete(disco_local_features, Host, ?MODULE, get_sm_features, 50),
     ejabberd_hooks:delete(webadmin_page_host, Host,
 			  ?MODULE, webadmin_page, 50),
     ejabberd_hooks:delete(webadmin_user, Host,
 			  ?MODULE, webadmin_user, 50),
+    ejabberd_hooks:delete(webadmin_user_parse_query, Host,
+                          ?MODULE, webadmin_user_parse_query, 50),
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     exit(whereis(Proc), stop),
     ok.
 
+get_sm_features(Acc, _From, _To, "", _Lang) ->
+    Feats = case Acc of
+		{result, I} -> I;
+		_ -> []
+	    end,
+    {result, Feats ++ [?NS_FEATURE_MSGOFFLINE]};
+
+get_sm_features(_Acc, _From, _To, ?NS_FEATURE_MSGOFFLINE, _Lang) ->
+    %% override all lesser features...
+    {result, []};
+
+get_sm_features(Acc, _From, _To, _Node, _Lang) ->
+    Acc.
+
+
 store_packet(From, To, Packet) ->
     Type = xml:get_tag_attr_s("type", Packet),
     if
 	(Type /= "error") and (Type /= "groupchat") and
 	(Type /= "headline") ->
-	    case check_event(From, To, Packet) of
+	    case check_event_chatstates(From, To, Packet) of
 		true ->
 		    #jid{luser = LUser} = To,
 		    TimeStamp = now(),
@@ -179,12 +221,22 @@ store_packet(From, To, Packet) ->
 	    ok
     end.
 
-check_event(From, To, Packet) ->
+%% Check if the packet has any content about XEP-0022 or XEP-0085
+check_event_chatstates(From, To, Packet) ->
     {xmlelement, Name, Attrs, Els} = Packet,
-    case find_x_event(Els) of
-	false ->
+    case find_x_event_chatstates(Els, {false, false, false}) of
+	%% There wasn't any x:event or chatstates subelements
+	{false, false, _} ->
 	    true;
-	El ->
+	%% There a chatstates subelement and other stuff, but no x:event
+	{false, CEl, true} when CEl /= false ->
+	    true;
+	%% There was only a subelement: a chatstates
+	{false, CEl, false} when CEl /= false ->
+	    %% Don't allow offline storage
+	    false;
+	%% There was an x:event element, and maybe also other stuff
+	{El, _, _} when El /= false ->
 	    case xml:get_subtag(El, "id") of
 		false ->
 		    case xml:get_subtag(El, "offline") of
@@ -206,22 +258,25 @@ check_event(From, To, Packet) ->
 					    {xmlelement, "offline", [], []}]}]
 					}),
 			    true
-			end;
+		    end;
 		_ ->
 		    false
 	    end
     end.
 
-find_x_event([]) ->
-    false;
-find_x_event([{xmlcdata, _} | Els]) ->
-    find_x_event(Els);
-find_x_event([El | Els]) ->
+%% Check if the packet has subelements about XEP-0022, XEP-0085 or other
+find_x_event_chatstates([], Res) ->
+    Res;
+find_x_event_chatstates([{xmlcdata, _} | Els], Res) ->
+    find_x_event_chatstates(Els, Res);
+find_x_event_chatstates([El | Els], {A, B, C}) ->
     case xml:get_tag_attr_s("xmlns", El) of
 	?NS_EVENT ->
-	    El;
+	    find_x_event_chatstates(Els, {El, B, C});
+	?NS_CHATSTATES ->
+	    find_x_event_chatstates(Els, {A, El, C});
 	_ ->
-	    find_x_event(Els)
+	    find_x_event_chatstates(Els, {A, B, true})
     end.
 
 find_x_expire(_, []) ->
@@ -321,7 +376,7 @@ user_queue(User, Server, Query, Lang) ->
     Username = ejabberd_odbc:escape(LUser),
     US = {LUser, LServer},
     Res = user_queue_parse_query(Username, LServer, Query),
-    Msgs = case catch ejabberd_odbc:sql_query(
+    MsgsAll = case catch ejabberd_odbc:sql_query(
 			LServer,
 			["select username, xml from spool"
 			 "  where username='", Username, "'"
@@ -339,6 +394,7 @@ user_queue(User, Server, Query, Lang) ->
 	       _ ->
 		   []
 	   end,
+    Msgs = get_messages_subset(User, Server, MsgsAll),
     FMsgs =
 	lists:map(
 	  fun({xmlelement, _Name, _Attrs, _Els} = Msg) ->
@@ -353,7 +409,7 @@ user_queue(User, Server, Query, Lang) ->
     [?XC("h1", io_lib:format(?T("~s's Offline Messages Queue"),
 			     [us_to_list(US)]))] ++
 	case Res of
-	    ok -> [?CT("Submitted"), ?P];
+	    ok -> [?XREST("Submitted")];
 	    nothing -> []
 	end ++
 	[?XAE("form", [{"action", ""}, {"method", "post"}],
@@ -425,11 +481,8 @@ user_queue_parse_query(Username, LServer, Query) ->
 us_to_list({User, Server}) ->
     jlib:jid_to_string({User, Server, ""}).
 
-webadmin_user(Acc, User, Server, Lang) ->
-    LUser = jlib:nodeprep(User),
-    LServer = jlib:nameprep(Server),
-    Username = ejabberd_odbc:escape(LUser),
-    QueueLen = case catch ejabberd_odbc:sql_query(
+get_queue_length(Username, LServer) ->
+    case catch ejabberd_odbc:sql_query(
 			    LServer,
 			    ["select count(*) from spool"
 			     "  where username='", Username, "';"]) of
@@ -437,9 +490,49 @@ webadmin_user(Acc, User, Server, Lang) ->
 		       SCount;
 		   _ ->
 		       0
-	       end,
+	       end.
+
+get_messages_subset(User, Host, MsgsAll) ->
+    Access = gen_mod:get_module_opt(Host, ?MODULE, access_max_user_messages,
+				    max_user_offline_messages),
+    MaxOfflineMsgs = case get_max_user_messages(Access, User, Host) of
+			 Number when is_integer(Number) -> Number;
+			 _ -> 100
+		     end,
+    Length = length(MsgsAll),
+    get_messages_subset2(MaxOfflineMsgs, Length, MsgsAll).
+
+get_messages_subset2(Max, Length, MsgsAll) when Length =< Max*2 ->
+    MsgsAll;
+get_messages_subset2(Max, Length, MsgsAll) ->
+    FirstN = Max,
+    {MsgsFirstN, Msgs2} = lists:split(FirstN, MsgsAll),
+    MsgsLastN = lists:nthtail(Length - FirstN - FirstN, Msgs2),
+    IntermediateMsg = {xmlelement, "...", [], []},
+    MsgsFirstN ++ [IntermediateMsg] ++ MsgsLastN.
+
+webadmin_user(Acc, User, Server, Lang) ->
+    LUser = jlib:nodeprep(User),
+    LServer = jlib:nameprep(Server),
+    Username = ejabberd_odbc:escape(LUser),
+    QueueLen = get_queue_length(Username, LServer),
     FQueueLen = [?AC("queue/", QueueLen)],
-    Acc ++ [?XCT("h3", "Offline Messages:")] ++ FQueueLen.
+    Acc ++ [?XCT("h3", "Offline Messages:")] ++ FQueueLen ++ [?C(" "), ?INPUTT("submit", "removealloffline", "Remove All Offline Messages")].
+
+webadmin_user_parse_query(_, "removealloffline", User, Server, _Query) ->
+    case catch odbc_queries:del_spool_msg(Server, User) of
+         {'EXIT', Reason} ->
+            ?ERROR_MSG("Failed to remove offline messages: ~p", [Reason]),
+            {stop, error};
+         {error, Reason} ->
+            ?ERROR_MSG("Failed to remove offline messages: ~p", [Reason]),
+            {stop, error};
+         _ ->
+            ?INFO_MSG("Removed all offline messages for ~s@~s", [User, Server]),
+            {stop, ok}
+    end;
+webadmin_user_parse_query(Acc, _Action, _User, _Server, _Query) ->
+    Acc.
 
 %% ------------------------------------------------
 %% mod_offline: number of messages quota management
