@@ -5,7 +5,7 @@
 %%% Created :  5 Oct 2006 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -317,7 +317,8 @@ process_active_set(LUser, LServer, {value, Name}) ->
 			    "match_presence_in", "match_presence_out"],
 		 RItems} ->
 		    Items = lists:map(fun raw_to_item/1, RItems),
-		    {result, [], #userlist{name = Name, list = Items}};
+		    NeedDb = is_list_needdb(Items),
+		    {result, [], #userlist{name = Name, list = Items, needdb = NeedDb}};
 		_ ->
 		    {error, ?ERR_INTERNAL_SERVER_ERROR}
 	    end;
@@ -518,7 +519,15 @@ parse_matches1(_Item, [{xmlelement, _, _, _} | _Els]) ->
 
 
 
-
+is_list_needdb(Items) ->
+    lists:any(
+      fun(X) ->
+	      case X#listitem.type of
+		  subscription -> true;
+		  group -> true;
+		  _ -> false
+	      end
+      end, Items).
 
 get_user_list(_, User, Server) ->
     LUser = jlib:nodeprep(User),
@@ -534,7 +543,8 @@ get_user_list(_, User, Server) ->
 			    "match_presence_in", "match_presence_out"],
 		 RItems} ->
 		    Items = lists:map(fun raw_to_item/1, RItems),
-		    #userlist{name = Default, list = Items};
+		    NeedDb = is_list_needdb(Items),
+		    #userlist{name = Default, list = Items, needdb = NeedDb};
 		_ ->
 		    #userlist{}
 	    end;
@@ -544,7 +554,7 @@ get_user_list(_, User, Server) ->
 
 
 check_packet(_, User, Server,
-	     #userlist{list = List},
+	     #userlist{list = List, needdb = NeedDb},
 	     {From, To, {xmlelement, PName, _, _}},
 	     Dir) ->
     case List of
@@ -560,33 +570,37 @@ check_packet(_, User, Server,
 		{message, in} ->
 		    LJID = jlib:jid_tolower(From),
 		    {Subscription, Groups} =
-			ejabberd_hooks:run_fold(
-			  roster_get_jid_info, jlib:nameprep(Server),
-			  {none, []}, [User, Server, LJID]),
+			case NeedDb of
+			    true -> ejabberd_hooks:run_fold(roster_get_jid_info, jlib:nameprep(Server), {none, []}, [User, Server, LJID]);
+			    false -> {[], []}
+			end,
 		    check_packet_aux(List, message,
 				     LJID, Subscription, Groups);
 		{iq, in} ->
 		    LJID = jlib:jid_tolower(From),
 		    {Subscription, Groups} =
-			ejabberd_hooks:run_fold(
-			  roster_get_jid_info, jlib:nameprep(Server),
-			  {none, []}, [User, Server, LJID]),
+			case NeedDb of
+			    true -> ejabberd_hooks:run_fold(roster_get_jid_info, jlib:nameprep(Server), {none, []}, [User, Server, LJID]);
+			    false -> {[], []}
+			end,
 		    check_packet_aux(List, iq,
 				     LJID, Subscription, Groups);
 		{presence, in} ->
 		    LJID = jlib:jid_tolower(From),
 		    {Subscription, Groups} =
-			ejabberd_hooks:run_fold(
-			  roster_get_jid_info, jlib:nameprep(Server),
-			  {none, []}, [User, Server, LJID]),
+			case NeedDb of
+			    true -> ejabberd_hooks:run_fold(roster_get_jid_info, jlib:nameprep(Server), {none, []}, [User, Server, LJID]);
+			    false -> {[], []}
+			end,
 		    check_packet_aux(List, presence_in,
 				     LJID, Subscription, Groups);
 		{presence, out} ->
 		    LJID = jlib:jid_tolower(To),
 		    {Subscription, Groups} =
-			ejabberd_hooks:run_fold(
-			  roster_get_jid_info, jlib:nameprep(Server),
-			  {none, []}, [User, Server, LJID]),
+			case NeedDb of
+			    true -> ejabberd_hooks:run_fold(roster_get_jid_info, jlib:nameprep(Server), {none, []}, [User, Server, LJID]);
+			    false -> {[], []}
+			end,
 		    check_packet_aux(List, presence_out,
 				     LJID, Subscription, Groups);
 		_ ->
@@ -769,132 +783,66 @@ item_to_raw(#listitem{type = Type,
     SMatchMessage = if MatchMessage -> "1"; true -> "0" end,
     SMatchPresenceIn = if MatchPresenceIn -> "1"; true -> "0" end,
     SMatchPresenceOut = if MatchPresenceOut -> "1"; true -> "0" end,
-    ["'", SType, "', "
-     "'", SValue, "', "
-     "'", SAction, "', "
-     "'", SOrder, "', "
-     "'", SMatchAll, "', "
-     "'", SMatchIQ, "', "
-     "'", SMatchMessage, "', "
-     "'", SMatchPresenceIn, "', "
-     "'", SMatchPresenceOut, "'"].
-
+    [SType, SValue, SAction, SOrder, SMatchAll, SMatchIQ,
+     SMatchMessage, SMatchPresenceIn, SMatchPresenceOut].
 
 sql_get_default_privacy_list(LUser, LServer) ->
     Username = ejabberd_odbc:escape(LUser),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["select name from privacy_default_list "
-       "where username='", Username, "';"]).
+    odbc_queries:get_default_privacy_list(LServer, Username).
 
 sql_get_default_privacy_list_t(LUser) ->
     Username = ejabberd_odbc:escape(LUser),
-    ejabberd_odbc:sql_query_t(
-      ["select name from privacy_default_list "
-       "where username='", Username, "';"]).
+    odbc_queries:get_default_privacy_list_t(Username).
 
 sql_get_privacy_list_names(LUser, LServer) ->
     Username = ejabberd_odbc:escape(LUser),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["select name from privacy_list "
-       "where username='", Username, "';"]).
+    odbc_queries:get_privacy_list_names(LServer, Username).
 
 sql_get_privacy_list_names_t(LUser) ->
     Username = ejabberd_odbc:escape(LUser),
-    ejabberd_odbc:sql_query_t(
-      ["select name from privacy_list "
-       "where username='", Username, "';"]).
+    odbc_queries:get_privacy_list_names_t(Username).
 
 sql_get_privacy_list_id(LUser, LServer, Name) ->
     Username = ejabberd_odbc:escape(LUser),
     SName = ejabberd_odbc:escape(Name),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["select id from privacy_list "
-       "where username='", Username, "' and name='", SName, "';"]).
+    odbc_queries:get_privacy_list_id(LServer, Username, SName).
 
 sql_get_privacy_list_id_t(LUser, Name) ->
     Username = ejabberd_odbc:escape(LUser),
     SName = ejabberd_odbc:escape(Name),
-    ejabberd_odbc:sql_query_t(
-      ["select id from privacy_list "
-       "where username='", Username, "' and name='", SName, "';"]).
+    odbc_queries:get_privacy_list_id_t(Username, SName).
 
 sql_get_privacy_list_data(LUser, LServer, Name) ->
     Username = ejabberd_odbc:escape(LUser),
     SName = ejabberd_odbc:escape(Name),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["select t, value, action, ord, match_all, match_iq, "
-       "match_message, match_presence_in, match_presence_out "
-       "from privacy_list_data "
-       "where id = (select id from privacy_list where "
-       "            username='", Username, "' and name='", SName, "') "
-       "order by ord;"]).
+    odbc_queries:get_privacy_list_data(LServer, Username, SName).
 
 sql_get_privacy_list_data_by_id(ID, LServer) ->
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["select t, value, action, ord, match_all, match_iq, "
-       "match_message, match_presence_in, match_presence_out "
-       "from privacy_list_data "
-       "where id='", ID, "' order by ord;"]).
+    odbc_queries:get_privacy_list_data_by_id(LServer, ID).
 
 sql_set_default_privacy_list(LUser, Name) ->
     Username = ejabberd_odbc:escape(LUser),
     SName = ejabberd_odbc:escape(Name),
-    ejabberd_odbc:sql_query_t(
-      ["delete from privacy_default_list "
-       "      where username='", Username, "';"]),
-    ejabberd_odbc:sql_query_t(
-      ["insert into privacy_default_list(username, name) "
-       "values ('", Username, "', '", SName, "');"]).
+    odbc_queries:set_default_privacy_list(Username, SName).
 
 sql_unset_default_privacy_list(LUser, LServer) ->
     Username = ejabberd_odbc:escape(LUser),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["delete from privacy_default_list "
-       "      where username='", Username, "';"]).
+    odbc_queries:unset_default_privacy_list(LServer, Username).
 
 sql_remove_privacy_list(LUser, Name) ->
     Username = ejabberd_odbc:escape(LUser),
     SName = ejabberd_odbc:escape(Name),
-    ejabberd_odbc:sql_query_t(
-      ["delete from privacy_list "
-       "where username='", Username, "' and name='", SName, "';"]).
+    odbc_queries:remove_privacy_list(Username, SName).
 
 sql_add_privacy_list(LUser, Name) ->
     Username = ejabberd_odbc:escape(LUser),
     SName = ejabberd_odbc:escape(Name),
-    ejabberd_odbc:sql_query_t(
-      ["insert into privacy_list(username, name) "
-       "values ('", Username, "', '", SName, "');"]).
+    odbc_queries:add_privacy_list(Username, SName).
 
 sql_set_privacy_list(ID, RItems) ->
-    ejabberd_odbc:sql_query_t(
-      ["delete from privacy_list_data "
-       "where id='", ID, "';"]),
-    lists:foreach(fun(Items) ->
-			  ejabberd_odbc:sql_query_t(
-			    ["insert into privacy_list_data("
-			     "id, t, value, action, ord, match_all, match_iq, "
-			     "match_message, match_presence_in, "
-			     "match_presence_out "
-			     ") "
-			     "values ('", ID, "', ", Items, ");"])
-		  end, RItems).
+    odbc_queries:set_privacy_list(ID, RItems).
 
 sql_del_privacy_lists(LUser, LServer) ->
     Username = ejabberd_odbc:escape(LUser),
     Server = ejabberd_odbc:escape(LServer),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["delete from privacy_list where username='", Username, "';"]),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["delete from privacy_list_data where value='", Username++"@"++Server, "';"]),
-    ejabberd_odbc:sql_query(
-      LServer,
-      ["delete from privacy_default_list where username='", Username, "';"]).
+    odbc_queries:del_privacy_lists(LServer, Server, Username).
