@@ -11,12 +11,12 @@
 %%% under the License.
 %%% 
 %%% The Initial Developer of the Original Code is ProcessOne.
-%%% Portions created by ProcessOne are Copyright 2006-2009, ProcessOne
+%%% Portions created by ProcessOne are Copyright 2006-2010, ProcessOne
 %%% All Rights Reserved.''
-%%% This software is copyright 2006-2009, ProcessOne.
+%%% This software is copyright 2006-2010, ProcessOne.
 %%%
 %%%
-%%% @copyright 2006-2009 ProcessOne
+%%% @copyright 2006-2010 ProcessOne
 %%% @author Christophe romain <christophe.romain@process-one.net>
 %%%   [http://www.process-one.net/]
 %%% @version {@vsn}, {@date} {@time}
@@ -41,51 +41,55 @@
 -export([init/3, terminate/2,
 	 options/0, features/0,
 	 create_node_permission/6,
-	 create_node/3,
-	 delete_node/2,
-	 purge_node/3,
+	 create_node/2,
+	 delete_node/1,
+	 purge_node/2,
 	 subscribe_node/8,
-	 unsubscribe_node/5,
-	 publish_item/7,
+	 unsubscribe_node/4,
+	 publish_item/6,
 	 delete_item/4,
-	 remove_extra_items/4,
+	 remove_extra_items/3,
 	 get_entity_affiliations/2,
-	 get_node_affiliations/2,
-	 get_affiliation/3,
-	 set_affiliation/4,
+	 get_node_affiliations/1,
+	 get_affiliation/2,
+	 set_affiliation/3,
 	 get_entity_subscriptions/2,
-	 get_node_subscriptions/2,
-	 get_subscription/3,
-	 set_subscription/4,
-	 get_states/2,
-	 get_state/3,
+	 get_node_subscriptions/1,
+	 get_subscriptions/2,
+	 set_subscriptions/4,
+	 get_pending_nodes/2,
+	 get_states/1,
+	 get_state/2,
 	 set_state/1,
-	 get_items/7,
+	 get_items/6,
 	 get_items/2,
-	 get_item/8,
-	 get_item/3,
-	 set_item/1
+	 get_item/7,
+	 get_item/2,
+	 set_item/1,
+	 get_item_name/3,
+	 node_to_path/1,
+	 path_to_node/1
 	]).
 
 
 init(Host, ServerHost, Opts) ->
-    node_default:init(Host, ServerHost, Opts).
+    node_hometree:init(Host, ServerHost, Opts).
 
 terminate(Host, ServerHost) ->
-    node_default:terminate(Host, ServerHost).
+    node_hometree:terminate(Host, ServerHost).
 
 options() ->
-    [{node_type, dispatch},
-     {deliver_payloads, true},
+    [{deliver_payloads, true},
      {notify_config, false},
      {notify_delete, false},
      {notify_retract, true},
      {persist_items, true},
-     {max_items, ?MAXITEMS div 2},
+     {max_items, ?MAXITEMS},
      {subscribe, true},
      {access_model, open},
      {roster_groups_allowed, []},
      {publish_model, publishers},
+     {notification_type, headline},
      {max_payload_size, ?MAX_PAYLOAD_SIZE},
      {send_last_published_item, never},
      {deliver_notifications, true},
@@ -108,81 +112,96 @@ features() ->
     ].
 
 create_node_permission(Host, ServerHost, Node, ParentNode, Owner, Access) ->
-    node_default:create_node_permission(Host, ServerHost, Node, ParentNode, Owner, Access).
+    node_hometree:create_node_permission(Host, ServerHost, Node, ParentNode, Owner, Access).
 
-create_node(Host, Node, Owner) ->
-    node_default:create_node(Host, Node, Owner).
+create_node(NodeId, Owner) ->
+    node_hometree:create_node(NodeId, Owner).
 
-delete_node(Host, Removed) ->
-    node_default:delete_node(Host, Removed).
+delete_node(Removed) ->
+    node_hometree:delete_node(Removed).
 
-subscribe_node(_Host, _Node, _Sender, _Subscriber, _AccessModel,
-	       _SendLast, _PresenceSubscription, _RosterGroup) ->
+subscribe_node(_NodeId, _Sender, _Subscriber, _AccessModel,
+	       _SendLast, _PresenceSubscription, _RosterGroup, _Options) ->
     {error, ?ERR_FORBIDDEN}.
 
-unsubscribe_node(_Host, _Node, _Sender, _Subscriber, _SubID) ->
+unsubscribe_node(_NodeId, _Sender, _Subscriber, _SubID) ->
     {error, ?ERR_FORBIDDEN}.
 
-publish_item(Host, Node, Publisher, Model, MaxItems, ItemId, Payload) ->
+publish_item(NodeId, Publisher, Model, MaxItems, ItemId, Payload) ->
     lists:foreach(fun(SubNode) ->
-			  node_default:publish_item(
-			    Host, SubNode, Publisher, Model,
+			  node_hometree:publish_item(
+			    SubNode#pubsub_node.id, Publisher, Model,
 			    MaxItems, ItemId, Payload)
-		  end, nodetree_default:get_subnodes(Host, Node)).
+		  end, nodetree_tree:get_subnodes(NodeId, Publisher, Publisher)).
 
-remove_extra_items(_Host, _Node, _MaxItems, ItemIds) ->
+remove_extra_items(_NodeId, _MaxItems, ItemIds) ->
     {result, {ItemIds, []}}.
 
-delete_item(_Host, _Node, _JID, _ItemId) ->
+delete_item(_NodeId, _Publisher, _PublishModel, _ItemId) ->
     {error, ?ERR_ITEM_NOT_FOUND}.
 
-purge_node(_Host, _Node, _Owner) ->
+purge_node(_NodeId, _Owner) ->
     {error, ?ERR_FORBIDDEN}.
 
 get_entity_affiliations(_Host, _Owner) ->
     {result, []}.
 
-get_node_affiliations(_Host, _Node) ->
+get_node_affiliations(_NodeId) ->
     {result, []}.
 
-get_affiliation(_Host, _Node, _Owner) ->
+get_affiliation(_NodeId, _Owner) ->
     {result, []}.
 
-set_affiliation(Host, Node, Owner, Affiliation) ->
-    node_default:set_affiliation(Host, Node, Owner, Affiliation).
+set_affiliation(NodeId, Owner, Affiliation) ->
+    node_hometree:set_affiliation(NodeId, Owner, Affiliation).
 
 get_entity_subscriptions(_Host, _Owner) ->
     {result, []}.
 
-get_node_subscriptions(_Host, _Node) ->
+get_node_subscriptions(NodeId) ->
+    %% note: get_node_subscriptions is used for broadcasting
+    %% DO NOT REMOVE
+    node_hometree:get_node_subscriptions(NodeId).
+
+get_subscriptions(_NodeId, _Owner) ->
     {result, []}.
 
-get_subscription(_Host, _Node, _Owner) ->
-    {result, []}.
+set_subscriptions(NodeId, Owner, Subscription, SubId) ->
+    node_hometree:set_subscriptions(NodeId, Owner, Subscription, SubId).
 
-set_subscription(Host, Node, Owner, Subscription) ->
-    node_default:set_subscription(Host, Node, Owner, Subscription).
+get_pending_nodes(Host, Owner) ->
+    node_hometree:get_pending_nodes(Host, Owner).
 
-get_states(Host, Node) ->
-    node_default:get_states(Host, Node).
+get_states(NodeId) ->
+    node_hometree:get_states(NodeId).
 
-get_state(Host, Node, JID) ->
-    node_default:get_state(Host, Node, JID).
+get_state(NodeId, JID) ->
+    node_hometree:get_state(NodeId, JID).
 
 set_state(State) ->
-    node_default:set_state(State).
+    node_hometree:set_state(State).
 
-get_items(Host, Node) ->
-    node_default:get_items(Host, Node).
+get_items(NodeId, From) ->
+    node_hometree:get_items(NodeId, From).
 
-get_items(Host, Node, JID, AccessModel, PresenceSubscription, RosterGroup, SubId) ->
-    node_default:get_items(Host, Node, JID, AccessModel, PresenceSubscription, RosterGroup, SubId).
+get_items(NodeId, JID, AccessModel, PresenceSubscription, RosterGroup, SubId) ->
+    node_hometree:get_items(NodeId, JID, AccessModel, PresenceSubscription, RosterGroup, SubId).
 
-get_item(Host, Node, ItemId) ->
-    node_default:get_item(Host, Node, ItemId).
+get_item(NodeId, ItemId) ->
+    node_hometree:get_item(NodeId, ItemId).
 
-get_item(Host, Node, ItemId, JID, AccessModel, PresenceSubscription, RosterGroup, SubId) ->
-    node_default:get_item(Host, Node, ItemId, JID, AccessModel, PresenceSubscription, RosterGroup, SubId).
+get_item(NodeId, ItemId, JID, AccessModel, PresenceSubscription, RosterGroup, SubId) ->
+    node_hometree:get_item(NodeId, ItemId, JID, AccessModel, PresenceSubscription, RosterGroup, SubId).
 
 set_item(Item) ->
-    node_default:set_item(Item).
+    node_hometree:set_item(Item).
+
+get_item_name(Host, Node, Id) ->
+    node_hometree:get_item_name(Host, Node, Id).
+
+node_to_path(Node) ->
+    node_flat:node_to_path(Node).
+
+path_to_node(Path) ->
+    node_flat:path_to_node(Path).
+
