@@ -5,7 +5,7 @@
 %%% Created : 15 Dec 2004 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -24,6 +24,15 @@
 %%%
 %%%----------------------------------------------------------------------
 
+%%% @doc Roster management (Mnesia storage).
+%%%
+%%% Includes support for XEP-0237: Roster Versioning.
+%%% The roster versioning follows an all-or-nothing strategy:
+%%%  - If the version supplied by the client is the latest, return an empty response.
+%%%  - If not, return the entire new roster (with updated version string).
+%%% Roster version is a hash digest of the entire roster.
+%%% No additional data is stored in DB.
+
 -module(mod_roster_odbc).
 -author('alexey@process-one.net').
 
@@ -41,7 +50,9 @@
 	 remove_user/2,
 	 get_jid_info/4,
 	 webadmin_page/3,
-	 webadmin_user/4]).
+	 webadmin_user/4,
+	 get_versioning_feature/2,
+	 roster_versioning_enabled/1]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
@@ -68,6 +79,8 @@ start(Host, Opts) ->
 		       ?MODULE, remove_user, 50),
     ejabberd_hooks:add(resend_subscription_requests_hook, Host,
 		       ?MODULE, get_in_pending_subscriptions, 50),
+    ejabberd_hooks:add(roster_get_versioning_feature, Host,
+		       ?MODULE, get_versioning_feature, 50),
     ejabberd_hooks:add(webadmin_page_host, Host,
 		       ?MODULE, webadmin_page, 50),
     ejabberd_hooks:add(webadmin_user, Host,
@@ -92,6 +105,8 @@ stop(Host) ->
 			  ?MODULE, remove_user, 50),
     ejabberd_hooks:delete(resend_subscription_requests_hook, Host,
 		       ?MODULE, get_in_pending_subscriptions, 50),
+    ejabberd_hooks:delete(roster_get_versioning_feature, Host,
+		          ?MODULE, get_versioning_feature, 50),
     ejabberd_hooks:delete(webadmin_page_host, Host,
 			  ?MODULE, webadmin_page, 50),
     ejabberd_hooks:delete(webadmin_user, Host,
@@ -99,25 +114,6 @@ stop(Host) ->
     gen_iq_handler:remove_iq_handler(ejabberd_sm, Host, ?NS_ROSTER).
 
 
--define(PSI_ROSTER_WORKAROUND, true).
-
--ifdef(PSI_ROSTER_WORKAROUND).
-
-process_iq(From, To, IQ) ->
-    #iq{sub_el = SubEl} = IQ,
-    #jid{lserver = LServer} = From,
-    case lists:member(LServer, ?MYHOSTS) of
-	true ->
-	    ResIQ = process_local_iq(From, To, IQ),
-	    ejabberd_router:route(From, From,
-				  jlib:iq_to_xml(ResIQ)),
-	    ignore;
-	_ ->
-	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_ITEM_NOT_FOUND]}
-    end.
-
--else.
-
 process_iq(From, To, IQ) ->
     #iq{sub_el = SubEl} = IQ,
     #jid{lserver = LServer} = From,
@@ -128,8 +124,6 @@ process_iq(From, To, IQ) ->
 	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_ITEM_NOT_FOUND]}
     end.
 
--endif.
-
 process_local_iq(From, To, #iq{type = Type} = IQ) ->
     case Type of
 	set ->
@@ -139,22 +133,101 @@ process_local_iq(From, To, #iq{type = Type} = IQ) ->
     end.
 
 
+roster_hash(Items) ->
+	sha:sha(term_to_binary(
+		lists:sort(
+			[R#roster{groups = lists:sort(Grs)} || 
+				R = #roster{groups = Grs} <- Items]))).
+		
+roster_versioning_enabled(Host) ->
+    gen_mod:get_module_opt(Host, ?MODULE, versioning, false).
 
+roster_version_on_db(Host) ->
+    gen_mod:get_module_opt(Host, ?MODULE, store_current_id, false).
+
+%% Returns a list that may contain an xmlelement with the XEP-237 feature if it's enabled.
+get_versioning_feature(Acc, Host) ->
+    case roster_versioning_enabled(Host) of
+	true ->
+	    Feature = {xmlelement,
+		       "ver",
+		       [{"xmlns", ?NS_ROSTER_VER}],
+		       [{xmlelement, "optional", [], []}]},
+	    [Feature | Acc];
+	false -> []
+    end.
+
+roster_version(LServer ,LUser) ->
+	US = {LUser, LServer},
+	case roster_version_on_db(LServer) of
+		true ->
+			case odbc_queries:get_roster_version(ejabberd_odbc:escape(LServer), ejabberd_odbc:escape(LUser)) of
+				{selected, ["version"], [{Version}]} -> Version;
+				{selected, ["version"], []} -> not_found
+			end;
+		false ->
+			roster_hash(ejabberd_hooks:run_fold(roster_get, LServer, [], [US]))
+	end.
+
+%% Load roster from DB only if neccesary. 
+%% It is neccesary if
+%%     - roster versioning is disabled in server OR
+%%     - roster versioning is not used by the client OR
+%%     - roster versioning is used by server and client, BUT the server isn't storing versions on db OR
+%%     - the roster version from client don't match current version.
 process_iq_get(From, To, #iq{sub_el = SubEl} = IQ) ->
     LUser = From#jid.luser,
     LServer = From#jid.lserver,
     US = {LUser, LServer},
-    case catch ejabberd_hooks:run_fold(roster_get, To#jid.lserver, [], [US]) of
-	Items when is_list(Items) ->
-	    XItems = lists:map(fun item_to_xml/1, Items),
-	    IQ#iq{type = result,
-		  sub_el = [{xmlelement, "query",
-			     [{"xmlns", ?NS_ROSTER}],
-			     XItems}]};
-	_ ->
-	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]}
+
+    try
+	    {ItemsToSend, VersionToSend} = 
+		case {xml:get_tag_attr("ver", SubEl), 
+		      roster_versioning_enabled(LServer),
+		      roster_version_on_db(LServer)} of
+		{{value, RequestedVersion}, true, true} ->
+			%% Retrieve version from DB. Only load entire roster
+			%% when neccesary.
+			case odbc_queries:get_roster_version(ejabberd_odbc:escape(LServer), ejabberd_odbc:escape(LUser)) of
+				{selected, ["version"], [{RequestedVersion}]} ->
+					{false, false};
+				{selected, ["version"], [{NewVersion}]} ->
+					{lists:map(fun item_to_xml/1, 
+						ejabberd_hooks:run_fold(roster_get, To#jid.lserver, [], [US])), NewVersion};
+				{selected, ["version"], []} ->
+					RosterVersion = sha:sha(term_to_binary(now())),
+					{atomic, {updated,1}} = odbc_queries:sql_transaction(LServer, fun() ->
+									odbc_queries:set_roster_version(ejabberd_odbc:escape(LUser), RosterVersion)
+								  end),
+
+					{lists:map(fun item_to_xml/1,
+						ejabberd_hooks:run_fold(roster_get, To#jid.lserver, [], [US])), RosterVersion}
+			end;
+
+		{{value, RequestedVersion}, true, false} ->
+			RosterItems = ejabberd_hooks:run_fold(roster_get, To#jid.lserver, [] , [US]),
+			case roster_hash(RosterItems) of
+				RequestedVersion ->
+					{false, false};
+				New ->
+					{lists:map(fun item_to_xml/1, RosterItems), New}
+			end;
+			
+		_ ->
+			{lists:map(fun item_to_xml/1, 
+					ejabberd_hooks:run_fold(roster_get, To#jid.lserver, [], [US])), false}
+		end,
+		IQ#iq{type = result, sub_el = case {ItemsToSend, VersionToSend} of
+					 	 {false, false} ->  [];
+						 {Items, false} -> [{xmlelement, "query", [{"xmlns", ?NS_ROSTER}], Items}];
+						 {Items, Version} -> [{xmlelement, "query", [{"xmlns", ?NS_ROSTER}, {"ver", Version}], Items}]
+						end}
+    catch 
+    	_:_ ->  
+		IQ#iq{type =error, sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]}
     end.
 
+
 get_user_roster(Acc, {LUser, LServer}) ->
     Items = get_roster(LUser, LServer),
     lists:filter(fun(#roster{subscription = none, ask = in}) ->
@@ -278,6 +351,7 @@ process_item_set(From, To, {xmlelement, _Name, Attrs, Els}) ->
 			Item2 = process_item_els(Item1, Els),
 			case Item2#roster.subscription of
 			    remove ->
+				io:format("del_roster: ~p ~p ~p \n", [LServer, Username, SJID]),
                                 odbc_queries:del_roster(LServer, Username, SJID);
 			    _ ->
 				ItemVals = record_to_string(Item2),
@@ -288,6 +362,10 @@ process_item_set(From, To, {xmlelement, _Name, Attrs, Els}) ->
 			%% subscription information from there:
 			Item3 = ejabberd_hooks:run_fold(roster_process_item,
 							LServer, Item2, [LServer]),
+			case roster_version_on_db(LServer) of
+				true -> odbc_queries:set_roster_version(ejabberd_odbc:escape(LUser), sha:sha(term_to_binary(now())));
+				false -> ok
+			end,
 			{Item, Item3}
 		end,
 	    case odbc_queries:sql_transaction(LServer, F) of
@@ -358,27 +436,17 @@ push_item(User, Server, From, Item) ->
 		       [{item,
 			 Item#roster.jid,
 			 Item#roster.subscription}]}),
-    lists:foreach(fun(Resource) ->
+    case roster_versioning_enabled(Server) of
+	true ->
+		push_item_version(Server, User, From, Item, roster_version(Server, User));
+	false ->
+	    lists:foreach(fun(Resource) ->
 			  push_item(User, Server, Resource, From, Item)
-		  end, ejabberd_sm:get_user_resources(User, Server)).
+		  end, ejabberd_sm:get_user_resources(User, Server))
+    end.
 
 % TODO: don't push to those who not load roster
--ifdef(PSI_ROSTER_WORKAROUND).
-
-push_item(User, Server, Resource, _From, Item) ->
-    ResIQ = #iq{type = set, xmlns = ?NS_ROSTER,
-		id = "push",
-		sub_el = [{xmlelement, "query",
-			   [{"xmlns", ?NS_ROSTER}],
-			   [item_to_xml(Item)]}]},
-    ejabberd_router:route(
-      jlib:make_jid(User, Server, Resource),
-      jlib:make_jid(User, Server, Resource),
-      jlib:iq_to_xml(ResIQ)).
-
--else.
-
-push_item(User, Resource, From, Item) ->
+push_item(User, Server, Resource, From, Item) ->
     ResIQ = #iq{type = set, xmlns = ?NS_ROSTER,
 		id = "push" ++ randoms:get_string(),
 		sub_el = [{xmlelement, "query",
@@ -389,7 +457,24 @@ push_item(User, Resource, From, Item) ->
       jlib:make_jid(User, Server, Resource),
       jlib:iq_to_xml(ResIQ)).
 
--endif.
+%% @doc Roster push, calculate and include the version attribute.
+%% TODO: don't push to those who didn't load roster
+push_item_version(Server, User, From, Item, RosterVersion)  ->
+    lists:foreach(fun(Resource) ->
+			  push_item_version(User, Server, Resource, From, Item, RosterVersion)
+		end, ejabberd_sm:get_user_resources(User, Server)).
+
+push_item_version(User, Server, Resource, From, Item, RosterVersion) ->
+    IQPush = #iq{type = 'set', xmlns = ?NS_ROSTER,
+		 id = "push" ++ randoms:get_string(),
+		 sub_el = [{xmlelement, "query",
+			    [{"xmlns", ?NS_ROSTER},
+			     {"ver", RosterVersion}],
+			    [item_to_xml(Item)]}]},
+    ejabberd_router:route(
+      From,
+      jlib:make_jid(User, Server, Resource),
+      jlib:iq_to_xml(IQPush)).
 
 get_subscription_lists(_, User, Server) ->
     LUser = jlib:nodeprep(User),
@@ -506,6 +591,10 @@ process_subscription(Direction, User, Server, JID1, Type, Reason) ->
 					      askmessage = AskMessage},
 			ItemVals = record_to_string(NewItem),
 			odbc_queries:roster_subscribe(LServer, Username, SJID, ItemVals),
+			case roster_version_on_db(LServer) of
+				true -> odbc_queries:set_roster_version(ejabberd_odbc:escape(LUser), sha:sha(term_to_binary(now())));
+				false -> ok
+			end,
 			{{push, NewItem}, AutoReply}
 		end
 	end,
@@ -699,9 +788,9 @@ set_items(User, Server, SubEl) ->
     LServer = jlib:nameprep(Server),
     catch odbc_queries:sql_transaction(
 	    LServer,
-	     lists:flatmap(fun(El) ->
-			       process_item_set_t(LUser, LServer, El)
-		       end, Els)).
+	    lists:flatmap(fun(El) ->
+				  process_item_set_t(LUser, LServer, El)
+			  end, Els)).
 
 process_item_set_t(LUser, LServer, {xmlelement, _Name, Attrs, Els}) ->
     JID1 = jlib:string_to_jid(xml:get_attr_s("jid", Attrs)),
@@ -915,13 +1004,7 @@ record_to_string(#roster{us = {User, _Server},
 	       none	   -> "N"
 	   end,
     SAskMessage = ejabberd_odbc:escape(AskMessage),
-    ["'", Username, "',"
-     "'", SJID, "',"
-     "'", Nick, "',"
-     "'", SSubscription, "',"
-     "'", SAsk, "',"
-     "'", SAskMessage, "',"
-     "'N', '', 'item'"].
+    [Username, SJID, Nick, SSubscription, SAsk, SAskMessage, "N", "", "item"].
 
 groups_to_string(#roster{us = {User, _Server},
 			 jid = JID,
@@ -931,12 +1014,11 @@ groups_to_string(#roster{us = {User, _Server},
 
     %% Empty groups do not need to be converted to string to be inserted in
     %% the database
-    lists:foldl(fun([], Acc) -> Acc;
-		   (Group, Acc) ->
-			String = ["'", Username, "',"
-				  "'", SJID, "',"
-				  "'", ejabberd_odbc:escape(Group), "'"],
-			[String|Acc] end, [], Groups).
+    lists:foldl(
+      fun([], Acc) -> Acc;
+	 (Group, Acc) ->
+ 	      G = ejabberd_odbc:escape(Group),
+	      [[Username, SJID, G]|Acc] end, [], Groups).
 
 webadmin_page(_, Host,
 	      #request{us = _US,
@@ -979,9 +1061,9 @@ user_roster(User, Server, Query, Lang) ->
 						  [?C(Group), ?BR]
 					  end, R#roster.groups),
 				    Pending = ask_to_pending(R#roster.ask),
+				    TDJID = build_contact_jid_td(R#roster.jid),
 				    ?XE("tr",
-					[?XAC("td", [{"class", "valign"}],
-					      jlib:jid_to_string(R#roster.jid)),
+					[TDJID,
 					 ?XAC("td", [{"class", "valign"}],
 					      R#roster.name),
 					 ?XAC("td", [{"class", "valign"}],
@@ -1008,8 +1090,8 @@ user_roster(User, Server, Query, Lang) ->
 	end,
     [?XC("h1", ?T("Roster of ") ++ us_to_list(US))] ++
 	case Res of
-	    ok -> [?CT("Submitted"), ?P];
-	    error -> [?CT("Bad format"), ?P];
+	    ok -> [?XREST("Submitted")];
+	    error -> [?XREST("Bad format")];
 	    nothing -> []
 	end ++
 	[?XAE("form", [{"action", ""}, {"method", "post"}],
@@ -1019,6 +1101,24 @@ user_roster(User, Server, Query, Lang) ->
 	       ?INPUTT("submit", "addjid", "Add Jabber ID")
 	      ])].
 
+build_contact_jid_td(RosterJID) ->
+    %% Convert {U, S, R} into {jid, U, S, R, U, S, R}:
+    ContactJID = jlib:make_jid(RosterJID),
+    JIDURI = case {ContactJID#jid.luser, ContactJID#jid.lserver} of
+		 {"", _} -> "";
+		 {CUser, CServer} ->
+		     case lists:member(CServer, ?MYHOSTS) of
+			 false -> "";
+			 true -> "/admin/server/" ++ CServer ++ "/user/" ++ CUser ++ "/"
+		     end
+	     end,
+    case JIDURI of
+	[] ->
+	    ?XAC("td", [{"class", "valign"}], jlib:jid_to_string(RosterJID));
+	URI when is_list(URI) ->
+	    ?XAE("td", [{"class", "valign"}], [?AC(JIDURI, jlib:jid_to_string(RosterJID))])
+    end.
+
 user_roster_parse_query(User, Server, Items, Query) ->
     case lists:keysearch("addjid", 1, Query) of
 	{value, _} ->
