@@ -5,7 +5,7 @@
 %%% Created :  2 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -96,7 +96,7 @@ stop(Host) ->
 
 get_sm_features({error, _Error} = Acc, _From, _To, _Node, _Lang) ->
     Acc;
- 
+
 get_sm_features(Acc, _From, _To, Node, _Lang) ->
     case Node of
 	[] ->
@@ -127,7 +127,7 @@ process_local_iq(_From, _To, #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ)
 				 translate:translate(
 				   Lang,
 				   "Erlang Jabber Server") ++
-				   "\nCopyright (c) 2002-2009 ProcessOne"}]},
+				   "\nCopyright (c) 2002-2010 ProcessOne"}]},
 			      {xmlelement, "BDAY", [],
 			       [{xmlcdata, "2002-11-16"}]}
 			     ]}]}
@@ -148,10 +148,7 @@ process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
 	get ->
 	    #jid{luser = LUser, lserver = LServer} = To,
 	    Username = ejabberd_odbc:escape(LUser),
-	    case catch ejabberd_odbc:sql_query(
-			 LServer,
-			 ["select vcard from vcard "
-			  "where username='", Username, "';"]) of
+	    case catch odbc_queries:get_vcard(LServer, Username) of
 		{selected, ["vcard"], [{SVCARD}]} ->
 		    case xml_stream:parse_element(SVCARD) of
 			{error, _Reason} ->
@@ -244,30 +241,13 @@ set_vcard(User, LServer, VCARD) ->
 	    SOrgUnit = ejabberd_odbc:escape(OrgUnit),
 	    SLOrgUnit = ejabberd_odbc:escape(LOrgUnit),
 
-	    ejabberd_odbc:sql_transaction(
-	      LServer,
-	      [["delete from vcard where username='", LUsername, "';"],
-	       ["insert into vcard(username, vcard) "
-	       "values ('", LUsername, "', '", SVCARD, "');"],
-	       ["delete from vcard_search where lusername='", LUsername, "';"],
-	       ["insert into vcard_search("
-		"        username, lusername, fn, lfn, family, lfamily,"
-		"        given, lgiven, middle, lmiddle, nickname, lnickname,"
-		"        bday, lbday, ctry, lctry, locality, llocality,"
-		"        email, lemail, orgname, lorgname, orgunit, lorgunit)"
-		"values (",
-		"        '", Username,  "', '", LUsername,  "'," 
-		"        '", SFN,       "', '", SLFN,       "'," 
-		"        '", SFamily,   "', '", SLFamily,   "',"
-		"        '", SGiven,    "', '", SLGiven,    "',"
-		"        '", SMiddle,   "', '", SLMiddle,   "',"
-		"        '", SNickname, "', '", SLNickname, "',"
-		"        '", SBDay,     "', '", SLBDay,	   "',"
-		"        '", SCTRY,     "', '", SLCTRY,	   "',"
-		"        '", SLocality, "', '", SLLocality, "',"
-		"        '", SEMail,    "', '", SLEMail,	   "',"
-		"        '", SOrgName,  "', '", SLOrgName,  "',"
-		"        '", SOrgUnit,  "', '", SLOrgUnit,  "');"]])
+	    odbc_queries:set_vcard(LServer, LUsername, SBDay, SCTRY, SEMail,
+				   SFN, SFamily, SGiven, SLBDay, SLCTRY,
+				   SLEMail, SLFN, SLFamily, SLGiven,
+				   SLLocality, SLMiddle, SLNickname,
+				   SLOrgName, SLOrgUnit, SLocality,
+				   SMiddle, SNickname, SOrgName,
+				   SOrgUnit, SVCARD, Username)
     end.
 
 -define(TLFIELD(Type, Label, Var),
@@ -364,6 +344,9 @@ do_route(ServerHost, From, To, Packet) ->
 				    Packet, ?ERR_NOT_ALLOWED),
 			    ejabberd_router:route(To, From, Err);
 			get ->
+			    Info = ejabberd_hooks:run_fold(
+				     disco_info, ServerHost, [],
+				     [ServerHost, ?MODULE, "", ""]),
 			    ResIQ =
 				IQ#iq{type = result,
 				      sub_el = [{xmlelement,
@@ -379,7 +362,7 @@ do_route(ServerHost, From, To, Packet) ->
 						   [{"var", ?NS_SEARCH}], []},
 						  {xmlelement, "feature",
 						   [{"var", ?NS_VCARD}], []}
-						 ]
+						 ] ++ Info
 						}]},
 			    ejabberd_router:route(To,
 						  From,
@@ -392,7 +375,7 @@ do_route(ServerHost, From, To, Packet) ->
 				    Packet, ?ERR_NOT_ALLOWED),
 			    ejabberd_router:route(To, From, Err);
 			get ->
-			    ResIQ = 
+			    ResIQ =
 				IQ#iq{type = result,
 				      sub_el = [{xmlelement,
 						 "query",
@@ -403,7 +386,7 @@ do_route(ServerHost, From, To, Packet) ->
 						  jlib:iq_to_xml(ResIQ))
 		    end;
 		#iq{type = get, xmlns = ?NS_VCARD, lang = Lang} ->
-		    ResIQ = 
+		    ResIQ =
 			IQ#iq{type = result,
 			      sub_el = [{xmlelement,
 					 "vCard",
@@ -428,7 +411,7 @@ iq_get_vcard(Lang) ->
       [{xmlcdata, translate:translate(
 		    Lang,
 		    "ejabberd vCard module") ++
-		    "\nCopyright (c) 2003-2009 ProcessOne"}]}].
+		    "\nCopyright (c) 2003-2010 ProcessOne"}]}].
 
 find_xdata_el({xmlelement, _Name, _Attrs, SubEls}) ->
     find_xdata_el1(SubEls).
@@ -635,18 +618,18 @@ make_val(Match, Field, Val) ->
 %	true ->
 %	    mnesia:write(
 %	      #vcard_search{us        = US,
-%			    user      = User,     luser      = LUser,     
-%			    fn        = FN,       lfn        = LFN,       
-%			    family    = Family,   lfamily    = LFamily,   
-%			    given     = Given,    lgiven     = LGiven,    
-%			    middle    = Middle,   lmiddle    = LMiddle,   
-%			    nickname  = Nickname, lnickname  = LNickname, 
-%			    bday      = BDay,     lbday      = LBDay,     
-%			    ctry      = CTRY,     lctry      = LCTRY,     
-%			    locality  = Locality, llocality  = LLocality, 
-%			    email     = EMail,    lemail     = LEMail,    
-%			    orgname   = OrgName,  lorgname   = LOrgName,  
-%			    orgunit   = OrgUnit,  lorgunit   = LOrgUnit   
+%			    user      = User,     luser      = LUser,
+%			    fn        = FN,       lfn        = LFN,
+%			    family    = Family,   lfamily    = LFamily,
+%			    given     = Given,    lgiven     = LGiven,
+%			    middle    = Middle,   lmiddle    = LMiddle,
+%			    nickname  = Nickname, lnickname  = LNickname,
+%			    bday      = BDay,     lbday      = LBDay,
+%			    ctry      = CTRY,     lctry      = LCTRY,
+%			    locality  = Locality, llocality  = LLocality,
+%			    email     = EMail,    lemail     = LEMail,
+%			    orgname   = OrgName,  lorgname   = LOrgName,
+%			    orgunit   = OrgUnit,  lorgunit   = LOrgUnit
 %			   })
 %    end.
 %
@@ -666,5 +649,3 @@ remove_user(User, Server) ->
       LServer,
       [["delete from vcard where username='", Username, "';"],
        ["delete from vcard_search where lusername='", Username, "';"]]).
-
-
