@@ -5,7 +5,7 @@
 %%% Created : 24 Jul 2004 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2009   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2010   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -159,7 +159,15 @@ recv(#tlssock{tcpsock = TCPSocket} = TLSSock,
 	    Error
     end.
 
-recv_data(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
+recv_data(TLSSock, Packet) ->
+    case catch recv_data1(TLSSock, Packet) of
+	{'EXIT', Reason} ->
+	    {error, Reason};
+	Res ->
+	    Res
+    end.
+
+recv_data1(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
     case port_control(Port, ?SET_ENCRYPTED_INPUT, Packet) of
 	<<0>> ->
 	    case port_control(Port, ?GET_DECRYPTED_INPUT, []) of
